package jdk.management.resource;

import java.util.Objects;
import java.util.HashSet;
import java.util.Set;
import java.util.WeakHashMap;

public class ResourceType
{
    private static final WeakHashMap<String, ResourceType> types;
    public static final ResourceType FILE_OPEN;
    public static final ResourceType FILE_READ;
    public static final ResourceType FILE_WRITE;
    public static final ResourceType STDERR_WRITE;
    public static final ResourceType STDIN_READ;
    public static final ResourceType STDOUT_WRITE;
    public static final ResourceType SOCKET_OPEN;
    public static final ResourceType SOCKET_READ;
    public static final ResourceType SOCKET_WRITE;
    public static final ResourceType DATAGRAM_OPEN;
    public static final ResourceType DATAGRAM_RECEIVED;
    public static final ResourceType DATAGRAM_SENT;
    public static final ResourceType DATAGRAM_READ;
    public static final ResourceType DATAGRAM_WRITE;
    public static final ResourceType THREAD_CREATED;
    public static final ResourceType THREAD_CPU;
    public static final ResourceType HEAP_RETAINED;
    public static final ResourceType HEAP_ALLOCATED;
    public static final ResourceType FILEDESCRIPTOR_OPEN;
    private final String name;
    private final boolean builtin;
    
    public static ResourceType of(String key) {
        synchronized (ResourceType.types) {
            return (ResourceType)ResourceType.types.computeIfAbsent(key, s/* java.lang.String, */ -> new ResourceType(s, false));
        }
    }
    
    static ResourceType ofBuiltin(String key) {
        synchronized (ResourceType.types) {
            return (ResourceType)ResourceType.types.computeIfAbsent(key, s/* java.lang.String, */ -> new ResourceType(s, true));
        }
    }
    
    private boolean isBuiltin() {
        return this.builtin;
    }
    
    static Set<ResourceType> builtinTypes() {
        synchronized (ResourceType.types) {
            HashSet set = new HashSet(ResourceType.types.values());
            set.removeIf(resourceType/* jdk.management.resource.ResourceType, */ -> !resourceType.isBuiltin());
            return set;
        }
    }
    
    private ResourceType(String obj, boolean builtin) {
        this.name = (String)Objects.requireNonNull(obj, "name");
        this.builtin = builtin;
        if (obj.length() == 0) {
            throw new IllegalArgumentException("name must not be empty");
        }
        return;
    }
    
    public String getName() {
        return this.name;
    }
    
    @Override
    public String toString() {
        return this.name;
    }
    
    @Override
    public int hashCode() {
        return 17 * 5 + Objects.hashCode(this.name);
    }
    
    @Override
    public boolean equals(Object o) {
        return o != null && this.getClass() == o.getClass() && Objects.equals(this.name, ((ResourceType)o).name);
    }
    
    static {
        types = new WeakHashMap(32);
        FILE_OPEN = ofBuiltin("file.open");
        FILE_READ = ofBuiltin("file.read");
        FILE_WRITE = ofBuiltin("file.write");
        STDERR_WRITE = ofBuiltin("stderr.write");
        STDIN_READ = ofBuiltin("stdin.read");
        STDOUT_WRITE = ofBuiltin("stdout.write");
        SOCKET_OPEN = ofBuiltin("socket.open");
        SOCKET_READ = ofBuiltin("socket.read");
        SOCKET_WRITE = ofBuiltin("socket.write");
        DATAGRAM_OPEN = ofBuiltin("datagram.open");
        DATAGRAM_RECEIVED = ofBuiltin("datagram.received");
        DATAGRAM_SENT = ofBuiltin("datagram.sent");
        DATAGRAM_READ = ofBuiltin("datagram.read");
        DATAGRAM_WRITE = ofBuiltin("datagram.write");
        THREAD_CREATED = ofBuiltin("thread.created");
        THREAD_CPU = ofBuiltin("thread.cpu");
        HEAP_RETAINED = ofBuiltin("heap.retained");
        HEAP_ALLOCATED = ofBuiltin("heap.allocated");
        FILEDESCRIPTOR_OPEN = ofBuiltin("filedescriptor.open");
    }
}
